/***********************************************************************************************************************
 *
 * blueBill Core - open source birding
 * Copyright (C) 2009-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://bluebill.tidalwave.it
 * SCM: https://kenai.com/hg/bluebill~core-src
 *
 **********************************************************************************************************************/
package it.tidalwave.bluebill.taxonomy;

import javax.annotation.Nonnull;

/***********************************************************************************************************************
 *
 * Implementations of this class will run a given {@link TaxonomyTraverser} on an {@link Taxonomy}. Each instance
 * of this class is bound to a given {@code Taxonomy}. Typically it can be created with code such as:
 * 
 * <pre>
 * Taxonomy taxonomy = ...
 * TaxonomyTraverser taxonomy = observationSet.as(TaxonomyTraverser);
 * traverser.accept(new TaxonomyVisitor() { ... });
 * </pre>
 * 
 * @navassoc - traverse - Taxonomy
 * @navassoc - call - TaxonomyVisitor
 * 
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
public interface TaxonomyTraverser
  {
    public static final Class<TaxonomyTraverser> TaxonomyTraverser = TaxonomyTraverser.class;
    
    public void accept (@Nonnull TaxonomyVisitor visitor);

    public void accept (@Nonnull Taxon taxon, final @Nonnull TaxonomyVisitor visitor);
  }
